@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -985,7 +985,8 @@ gl3::SceneInfo VisualizationSceneVector::GetSceneObjs()
    {
       scene.queue.emplace_back(params, &vector_buf);
    }
-   params.contains_translucent = matAlpha < 1.0;
+   params.contains_translucent = matAlpha < 1.0 ||
+                                 palette.GetPalette()->IsTranslucent();
    if (drawelems)
    {
       scene.queue.emplace_back(params, &disp_buf);
