@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-443271.
 //
@@ -404,6 +404,15 @@ void VisualizationSceneVector3d::Init()
 
    VisualizationSceneSolution3d::Init();
 
+   mesh_volume = 0.0;
+   if (mesh)
+   {
+      for (int i=0; i<mesh->GetNE(); i++)
+      {
+         mesh_volume += mesh->GetElementVolume(i);
+      }
+   }
+
    PrepareVectorField();
    PrepareDisplacedMesh();
 
@@ -1333,7 +1342,8 @@ void VisualizationSceneVector3d::DrawVector(gl3::GlDrawable& buf,
                                             double sz, double s)
 {
    static int nv = mesh -> GetNV();
-   static double volume = (bb.x[1]-bb.x[0])*(bb.y[1]-bb.y[0])*(bb.z[1]-bb.z[0]);
+   static double bb_vol = (bb.x[1]-bb.x[0])*(bb.y[1]-bb.y[0])*(bb.z[1]-bb.z[0]);
+   static double volume = std::max(bb_vol, mesh_volume);
    static double h      = pow(volume/nv, 0.333);
    static double hh     = pow(volume, 0.333) / 10;
 
@@ -1646,7 +1656,8 @@ gl3::SceneInfo VisualizationSceneVector3d::GetSceneObjs()
    params.use_clip_plane = cplane;
    double* cp_eqn = CuttingPlane->Equation();
    params.clip_plane_eqn = {cp_eqn[0], cp_eqn[1], cp_eqn[2], cp_eqn[3]};
-   params.contains_translucent = matAlpha < 1.0;
+   params.contains_translucent = matAlpha < 1.0 ||
+                                 palette.GetPalette()->IsTranslucent();
    // draw vector field
    if (drawvector == 2 || drawvector == 3)
    {
